// SDL2 Demo by aurelien.esnard@u-bordeaux.fr

#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>           // required to load transparent texture from PNG
#include <SDL2/SDL_ttf.h>             // required to use TTF fonts

#include <stdio.h>
#include <stdbool.h>

/* **************************************************************** */

#ifdef __ANDROID__ 
#define PRINT(STR, ...) do { SDL_Log(STR, ##__VA_ARGS__);  } while(0)
#define ERROR(STR, ...) do { SDL_Log(STR, ##__VA_ARGS__); exit(EXIT_FAILURE); } while(0) 
// #define LOG(TAG, STR, ...) __android_log_print(ANDROID_LOG_VERBOSE, TAG ,STR, ##__VA_ARGS__)
# else
#define PRINT(STR, ...) do { printf(STR, ##__VA_ARGS__); } while(0)
#define ERROR(STR, ...) do { fprintf(stderr, STR, ##__VA_ARGS__); exit(EXIT_FAILURE); } while(0)
#endif

/* **************************************************************** */

#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 480
#define FONT "Arial.ttf"
#define FONTSIZE 36
#define BACKGROUND "background.png"
#define MARIO "mario.png"
#define BOMB "bomb.png"

/* **************************************************************** */

/* your main environment structure */
typedef struct  {
  SDL_Texture * background;
  SDL_Texture * mario;
  SDL_Texture * bomb;  
  SDL_Texture * text;
  int bomb_x, bomb_y;
  int mario_x, mario_y;  
} Env; 

/* **************************************************************** */

void init(SDL_Window* win, SDL_Renderer* ren, Env * env)
{
  PRINT("Move the bomb with keyboard arrows and move Mario with mouse.\n");
  PRINT("Press ESC to quit. Enjoy this SDL2 sample!\n");  
  
  /* get current window size */
  int w, h;
  SDL_GetWindowSize(win, &w, &h);

  /* init positions */
  env->bomb_x = w/2;
  env->bomb_y = h/4;
  env->mario_x = w/2;
  env->mario_y = 3*h/4;
  
  /* init background texture from PNG image */
  env->background = IMG_LoadTexture(ren, BACKGROUND);
  if(!env->background) ERROR("IMG_LoadTexture: %s\n", BACKGROUND);
  
  /* init mario texture from PNG image */
  env->mario = IMG_LoadTexture(ren, MARIO);
  if(!env->mario) ERROR("IMG_LoadTexture: %s\n", MARIO);

  /* init bomb texture from PNG image */
  env->bomb = IMG_LoadTexture(ren, BOMB);
  if(!env->bomb) ERROR("IMG_LoadTexture: %s\n", BOMB);

  /* init text texture using Arial font */
  SDL_Color color = { 0, 0, 255, 255 }; /* blue color in RGBA */
  TTF_Font * font = TTF_OpenFont(FONT, FONTSIZE);
  if(!font) ERROR("TTF_OpenFont: %s\n", FONT);
  TTF_SetFontStyle(font, TTF_STYLE_BOLD); // TTF_STYLE_ITALIC | TTF_STYLE_NORMAL
  SDL_Surface * surf = TTF_RenderText_Blended(font, "Hello World!", color); // blended rendering for ultra nice text
  env->text = SDL_CreateTextureFromSurface(ren, surf);
  SDL_FreeSurface(surf);
  TTF_CloseFont(font);
}

/* **************************************************************** */

void render(SDL_Window* win, SDL_Renderer* ren, Env * env)
{
  SDL_Rect rect;  
 
  /* get current window size */
  int w, h;
  SDL_GetWindowSize(win, &w, &h);
    
  /* render background texture */
  SDL_RenderCopy(ren, env->background, NULL, NULL); /* stretch it */

  /* render text texture */
  SDL_QueryTexture(env->text, NULL, NULL, &rect.w, &rect.h);
  rect.x = w/2 - rect.w/2; rect.y = h/2 - rect.h/2; 
  SDL_RenderCopy(ren, env->text, NULL, &rect);

  /* render mario texture */
  SDL_QueryTexture(env->mario, NULL, NULL, &rect.w, &rect.h);
  rect.x = env->mario_x - rect.w/2; rect.y = env->mario_y - rect.h/2; 
  SDL_RenderCopy(ren, env->mario, NULL, &rect); 

  /* render bomb texture */
  SDL_QueryTexture(env->bomb, NULL, NULL, &rect.w, &rect.h);
  rect.x = env->bomb_x - rect.w/2; rect.y = env->bomb_y - rect.h/2; 
  SDL_RenderCopy(ren, env->bomb, NULL, &rect);   
}

/* **************************************************************** */

void clean(SDL_Window* win, SDL_Renderer* ren, Env * env)
{
  SDL_DestroyTexture(env->background);
  SDL_DestroyTexture(env->mario);
  SDL_DestroyTexture(env->bomb);  
  SDL_DestroyTexture(env->text);
}

/* **************************************************************** */

bool process(SDL_Window* win, SDL_Renderer* ren, Env * env, SDL_Event * e)
{      
  int w, h;
  SDL_GetWindowSize(win, &w, &h);  

  /* generic events */  
  if (e->type == SDL_QUIT) {
    return true;
  }
  /* Android events */  
#ifdef __ANDROID__ 
  else if (e->type == SDL_FINGERDOWN) {    
    env->mario_x = e->tfinger.x * w; /* tfinger.x, normalized in [0..1] */
    env->mario_y = e->tfinger.y * h; /* tfinger.y, normalized in [0..1] */
  }
  /* other events */  
#else
  else if (e->type == SDL_MOUSEBUTTONDOWN) {
    SDL_Point mouse; 
    SDL_GetMouseState(&mouse.x, &mouse.y);    
    env->mario_x = mouse.x;
    env->mario_y = mouse.y;    
  }
  else if (e->type == SDL_KEYDOWN) {
    switch (e->key.keysym.sym) {
    case SDLK_LEFT:  env->bomb_x -= 10; break;
    case SDLK_RIGHT: env->bomb_x += 10; break;
    case SDLK_UP:    env->bomb_y -= 10; break;
    case SDLK_DOWN:  env->bomb_y += 10; break;
    case SDLK_ESCAPE:  return true; break;      
    }
  }  
#endif  

  return false; /* don't quit */
}

/* **************************************************************** */

int main(int argc, char * argv[])
{ 
  /* initialize SDL2 and some extensions */
  if(SDL_Init(SDL_INIT_VIDEO) != 0) ERROR("SDL_Init VIDEO"); 
  if(IMG_Init(IMG_INIT_PNG & IMG_INIT_PNG) != IMG_INIT_PNG) ERROR("IMG_Init PNG");
  if(TTF_Init() != 0) ERROR("TTF_Init");  

  /* create window and renderer */
  SDL_Window * win = SDL_CreateWindow("Hello World!", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
				      SCREEN_WIDTH, SCREEN_HEIGHT, SDL_WINDOW_SHOWN | SDL_WINDOW_RESIZABLE);
  if(!win) ERROR("SDL_CreateWindow");  
  SDL_Renderer * ren = SDL_CreateRenderer(win, -1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC);
  if(!ren) ERROR("SDL_CreateWindow");    

  /* initialize your environment */
  Env env;
  init(win, ren, &env);
  
  /* main render loop */
  bool quit = false;
  while (!quit) {

    /* manage events */
    SDL_Event e;
    while (SDL_PollEvent(&e)) {
      /* process your events */
      quit = process(win, ren, &env, &e);
      if(quit) break;
    }
    

    /* background in gray */
    SDL_SetRenderDrawColor(ren, 0xA0, 0xA0, 0xA0, 0xFF); 
    SDL_RenderClear(ren);

    /* render all what you want */    
    render(win, ren, &env); 
    SDL_RenderPresent(ren);
    SDL_Delay(10);
  }
  
  /* clean your environment */
  clean(win, ren, &env);
  
  SDL_DestroyRenderer(ren);
  SDL_DestroyWindow(win);
  IMG_Quit();
  TTF_Quit();  
  SDL_Quit();

  return EXIT_SUCCESS;
}
